import sys
from argparse import ArgumentParser

from lightroom_export_organizer import greet_world


def get_clargs():
    parser = ArgumentParser()
    parser.add_argument("-d", "--descriptor", help="Descriptor of world to greet.")
    return parser.parse_args()


def main():
    args = get_clargs()
    sys.exit(greet_world(args.descriptor))



import pytest

from lightroom_export_organizer.greet_world import greet_world


@pytest.mark.parametrize("descriptor", [
    (None),
    ("Big")
])
def test_dummy(descriptor):
    greet_world(descriptor)



"""
Organizes exported Lightroom photos into directories, according to a key-value pair in
a side-car text file for each image. The value of this key-value pair is the name of the
directory that the photo should be put into.

UI code is a liberal adaptation of https://pythonspot.com/pyqt5-file-dialog/
"""
import os
import sys
import structlog
from os import path
from argparse import ArgumentParser
from PyQt5.QtWidgets import QApplication, QWidget, QFileDialog

from lightroom_export_organizer.org_folders import do2

log = structlog.getLogger()


class UI(QWidget):

    def __init__(self):
        super().__init__()
        self.title = "Lightroom Export Organizer"
        self.init_ui()

    def get_directory(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        return QFileDialog.getExistingDirectory(
            self,
            "Select the directory where the photos are.",
            path.join(path.expanduser("~"), "Desktop"),
            options=options
        )

    def init_ui(self):
        self.setWindowTitle(self.title)


def get_clargs():
    parser = ArgumentParser()
    parser.add_argument("-i", "--input-directory", required=False, default=None,
                        help="Path to directory")
    return parser.parse_args()


def main():
    args = get_clargs()
    os.environ["DISPLAY"] = ":0.0"
    while args.input_directory is None:
        app = QApplication(sys.argv)
        args.input_directory = UI().get_directory()

    return do2(args.input_directory)


if __name__ == '__main__':
    sys.exit(main())



import os
import shutil
import structlog
from os import path
from glob import glob
import numpy as np
from collections import defaultdict

log = structlog.getLogger()


def validate_file_pairs(base_directory):
    """
    - Make directories
    - Delete resulting empty directories
    - Make a backup of the photos
    - Send the backup to a local directory
    - Send the report to a new sub-domain of anthonyagnone.com

    Report Structure
        - list of the file moves
        - list of the created directories
        - list of the deleted directories
        - list of file base names which did not have a {.jpg, .txt} pair

    :param str base_directory:
        Base directory to search for images under.
    :return tuple:
        bool: indication of success
        msg: supporting description message
    """

    paths_valid = []
    paths_invalid = []
    for root, dirs, files in os.walk(base_directory):
        for fn in files:
            base, ext = path.splitext(fn)
            # for each assumed {.jpg, .txt} pair, use .jpg as the anchor file for looking for the
            # other two.
            if ext.lower() in (".jpg", ".jpeg"):
                base_path = path.join(root, base)
                if path.exists("{}.txt".format(base_path)):
                    log.debug("{} is a valid pair.".format(base_path))
                    paths_valid.append(base_path)
                else:
                    log.error("{} is not a valid pair.".format(base_path))
                    paths_invalid.append(base_path)

    return paths_valid, paths_invalid


def read_keyword(fn):
    keyword = None
    with open(fn, 'r') as fp:
        for line in fp:
            if "beeb" in line and ":" in line:
                keyword = line.split(":")[1].strip()
    return keyword


def file_parts(fn_path):
    base_dir = path.dirname(fn_path)
    base_fn = path.basename(fn_path)
    return [base_dir] + list(path.splitext(base_fn))


def make_file_list(dir_files):
    return [path.join(root, f) for root, _, files in os.walk(dir_files) for f in files]


def parse_file_pairs(file_list):
    pairs = defaultdict(dict)
    for f in file_list:
        base_dir, base_fn, ext = file_parts(f)
        key = path.join(base_dir, base_fn)
        pairs[key][ext] = True
    return pairs


def lower_extension(fn):
    base_dir, base_fn, ext = file_parts(fn)
    return path.join(base_dir, base_fn) + ext.lower()


def lower_extensions(dir_base):
    for root, _, files in os.walk(dir_base):
        for f in files:
            fn_old = path.join(root, f)
            fn_new = lower_extension(fn_old)
            if fn_old != fn_new:
                shutil.move(fn_old, fn_new)


def ensure_dir(dir_name):
    if not path.isdir(dir_name):
        os.makedirs(dir_name)


def move_files(files, directory):
    """
    Move an unknown/invalid file set to the designated unknown directory.
    :param files: files to move
    :param directory: destination directory
    :return :
    """
    ensure_dir(directory)

    for f in files:
        dir_name, base_fn, ext = file_parts(f)
        log.msg("symlink created: {} -> {}".format(f, path.join(directory, base_fn) + ext))
        os.symlink(
            f,
            path.join(directory, base_fn) + ext
        )


def do2(dir_base):

    lower_extensions(dir_base)
    file_list = make_file_list(dir_base)
    file_pairs = parse_file_pairs(file_list)

    for base_fn, ext_map in file_pairs.iteritems():
        if ext_map.pop('.txt', False):
            # try to parse the keyword from the txt file
            keyword = read_keyword(base_fn + '.txt')
            if keyword:
                # check for existence of the .jpg
                if ext_map.pop('.jpg', False):
                    # move the txt and jpg file to the validated directory
                    move_files(
                        [base_fn + '.txt', base_fn + '.jpg'],
                        path.join(dir_base, keyword))

        # if any other extensions are remaining for this file group, move to unknowns
        if len(ext_map) > 0:
            move_files(
                [base_fn + ext for ext in ext_map.iterkeys()],
                path.join(dir_base, "unknown"))


def do(dir_base):
    """
    - Make directories
    - Delete resulting empty directories
    - Make a backup of the photos
    - Send the backup to a local directory
    - Send the report to a new sub-domain of anthonyagnone.com

    Report Structure
        - list of the file moves
        - list of the created directories
        - list of the deleted directories
        - list of file base names which did not have a {.jpg, .txt} pair

    :param str dir_base:
        Base directory to search for images under.
    :return tuple:
        bool: indication of success
        msg: supporting description message
    """

    dir_unknowns = path.join(dir_base, 'unknown')
    paths_moved = []
    dirs_created = []
    dirs_removed = []
    keywords = set()

    fns_valid, fns_invalid = validate_file_pairs(dir_base)

    # report any invalid pairs
    if len(fns_invalid) > 0:
        log.error("{} invalid file pairs detected.".format(len(fns_invalid)))
        if not path.isdir(dir_unknowns):
            os.mkdir(dir_unknowns)

    for fn in np.unique(fns_valid):
        keyword = read_keyword(fn + ".txt")

        if keyword:
            dir_keyword = path.join(dir_base, keyword)

            # if this is a new keyword, create its directory
            if keyword not in keywords:
                log.msg("New keyword found: {}. Creating directory".format(keyword))
                if not path.isdir(dir_keyword):
                    os.mkdir(dir_keyword)
                dirs_created.append(dir_keyword)
                keywords.add(keyword)

            # move the file into the keyword directory
            log.msg("Moving {} to {}.".format(fn, dir_keyword))
            for file_to_move in glob(fn + '.*'):
                shutil.move(file_to_move, dir_keyword)
            os.remove(path.join(dir_keyword, path.basename(fn) + ".txt"))
        else:
            # keyword not successfully found -- add to unknowns
            for file_to_move in glob(fn + '.*'):
                shutil.move(file_to_move, path.join(dir_unknowns, path.basename(fn)))

    for fn in fns_invalid:
        log.msg("Moving {} to {}.".format(fn, dir_unknowns))
        for file_to_move in glob(fn + '.*'):
            shutil.move(file_to_move, dir_unknowns)

    # remove any empty directories that the file movement creates
    for root, dirs, files in os.walk(dir_base):
        for d in dirs:
            dir_cur = path.join(root, d)
            if is_empty_directory(dir_cur):
                log.msg("Detected empty directory {}. Removing.".format(dir_cur))
                dirs_removed.append(dir_cur)
                os.rmdir(dir_cur)

    return 0, "Success"


def is_empty_directory(dir_name):
    return len(os.listdir(dir_name)) == 0


def undo():
    pass



import os
import shutil
import pytest
import tempfile
from os import path
from collections import defaultdict
from tempfile import NamedTemporaryFile

from lightroom_export_organizer.org_folders import (
    do,
    do2,
    undo,
    validate_file_pairs,
    read_keyword,
    make_file_list,
    file_parts,
    parse_file_pairs,
    lower_extensions,
    lower_extension
)


@pytest.fixture(scope='session')
def fixture_dir():
    return path.join(path.dirname(__file__), 'fixtures')


@pytest.fixture(scope="function")
def fail_case_file_list(fixture_dir):
    with open(fixture_dir, 'r') as fp:
        lines = [line for line in fp.readlines()]
    return lines


@pytest.fixture(scope="function")
def test_fn():
    with tempfile.NamedTemporaryFile('w', delete=False) as fp:
        fp.write("beeb: dest1\n")
    yield fp.name

    if path.isfile(fp.name):
        os.remove(fp.name)


@pytest.fixture(scope="function")
def test_info(tmpdir_factory):

    def touch(fn, keyword):
        with open(fn, 'a') as fp:
            fp.write("beeb: {}\n".format(keyword))

    def make_pair(base_fn, keyword, bad=False):
        touch(base_fn + ".jpg", keyword)
        if not bad:
            touch(base_fn + ".txt", keyword)

    dir_base = str(tmpdir_factory.mktemp("test_dir_base"))
    tmpdir_maker = defaultdict(lambda: path.join(dir_base, str(tmpdir_factory.mktemp("test_dirs"))))

    make_pair(path.join(dir_base, "b1"), "dest1")
    make_pair(path.join(dir_base, "b2"), "dest2")
    make_pair(path.join(dir_base, "b3"), "dest3", bad=True)

    d1 = path.join(dir_base, "d1")
    os.mkdir(d1)
    make_pair(path.join(d1, "f1"), "dest1")
    make_pair(path.join(d1, "f2"), "dest2")
    make_pair(path.join(d1, "f3"), "dest1", bad=True)

    d2 = path.join(dir_base, "d2")
    os.mkdir(d2)
    make_pair(path.join(d2, "f4"), "dest3")
    make_pair(path.join(d2, "f5"), "dest2", bad=True)

    validity_map = {
        "valid": {
            'b1': (path.join(dir_base, 'b1'), 'dest1'),
            'b2': (path.join(dir_base, 'b2'), 'dest2'),
            'f1': (path.join(d1, "f1"), "dest1"),
            'f2': (path.join(d1, "f2"), "dest2"),
            'f4': (path.join(d1, "f4"), "dest3")
        },
        "invalid": {
            'b3': (path.join(dir_base, 'b3'), 'dest3'),
            'f3': (path.join(d1, "f3"), "dest1"),
            'f5': (path.join(d1, "f5"), "dest2")
        }
    }

    file_list = [
        path.join(dir_base, 'b1.txt'),
        path.join(dir_base, 'b1.jpg'),
        path.join(dir_base, 'b2.txt'),
        path.join(dir_base, 'b2.jpg'),
        path.join(dir_base, 'b3.jpg'),  # .txt missing because bad
        path.join(d1, 'f1.txt'),
        path.join(d1, 'f1.jpg'),
        path.join(d1, 'f2.txt'),
        path.join(d1, 'f2.jpg'),
        path.join(d1, 'f3.jpg'),  # .txt missing because bad
        path.join(d2, 'f4.txt'),
        path.join(d2, 'f4.jpg'),
        path.join(d2, 'f5.jpg')  # .txt missing because bad
    ]

    yield dir_base, validity_map, file_list

    if path.isdir(dir_base):
        shutil.rmtree(dir_base)
    for dir_name in tmpdir_maker.values():
        if path.isdir(dir_name):
            shutil.rmtree(dir_name)


def test_do(test_info, tmpdir_factory):
    dir_base, validity_map, expected_file_list = test_info

    do(dir_base)

    for base_name, (old_path, keyword) in validity_map['valid'].items():
        assert path.isdir(path.join(dir_base, keyword))
        assert path.isfile(path.join(dir_base, keyword, base_name) + ".jpg")
        # assert not path.isfile(path.join(dir_base, keyword, base_name) + ".txt")

    for base_name, (old_path, keyword) in validity_map['invalid'].items():
        assert path.isdir(path.join(dir_base, 'unknown'))
        assert path.isfile(path.join(dir_base, 'unknown', base_name) + ".jpg")


def test_do2(test_info, tmpdir_factory):
    dir_base, validity_map, expected_file_list = test_info

    do2(dir_base)

    for base_name, (old_path, keyword) in validity_map['valid'].items():
        assert path.isdir(path.join(dir_base, keyword))
        assert path.isfile(path.join(dir_base, keyword, base_name) + ".jpg")
        # assert not path.isfile(path.join(dir_base, keyword, base_name) + ".txt")

    for base_name, (old_path, keyword) in validity_map['invalid'].items():
        assert path.isdir(path.join(dir_base, 'unknown'))
        assert path.isfile(path.join(dir_base, 'unknown', base_name) + ".jpg")


def test_read_keyword(test_fn):
    assert "dest1" == read_keyword(test_fn)

    with NamedTemporaryFile(delete=False) as fp:
        fn = fp.name
    keyword = read_keyword(fn)
    os.remove(fn)
    assert keyword is None


def test_validate_file_pairs(test_info):
    test_directory, validity_map, expected_file_list = test_info
    valid, invalid = validate_file_pairs(test_directory)

    def get_base_fn(fn):
        return path.splitext(path.basename(fn))[0]

    valid = list(map(get_base_fn, valid))
    invalid = list(map(get_base_fn, invalid))

    for item_valid in validity_map['valid'].keys():
        assert item_valid in valid
        assert item_valid not in invalid

    for item_invalid in validity_map['invalid'].keys():
        assert item_invalid in invalid
        assert item_invalid not in valid


@pytest.mark.parametrize("in_fn,expected", [
    ("/path/To/a.txt", "/path/To/a.txt"),
    ("/path/To/a.TXT", "/path/To/a.txt"),
])
def test_lower_extension(in_fn, expected):
    assert expected == lower_extension(in_fn)


@pytest.fixture(scope='function')
def extension_test_dir(tmpdir_factory):
    dir_base = str(tmpdir_factory.mktemp("test_extensions"))

    def touch(fn):
        with open(fn, 'w') as fp:
            fp.write('hello\n')

    before = [
        path.join(dir_base, "a.txt"),
        path.join(dir_base, "a.jpg"),
        path.join(dir_base, "a.TXT"),
        path.join(dir_base, "a.JPG"),
        path.join(dir_base, "a.MP3")
    ]

    after = [
        path.join(dir_base, "a.txt"),
        path.join(dir_base, "a.jpg"),
        path.join(dir_base, "a.txt"),
        path.join(dir_base, "a.jpg"),
        path.join(dir_base, "a.mp3")
    ]

    # create all the files
    map(touch, before)

    yield dir_base, before, after

    # clean up
    if path.isdir(dir_base):
        shutil.rmtree(dir_base)


def test_lower_extensions(extension_test_dir):
    dir_name, before, after = extension_test_dir
    lower_extensions(dir_name)
    assert all(map(os.path.isfile, after))


def test_make_file_list(test_info):
    dir_base, _, expected_file_list = test_info

    actual_file_list = make_file_list(dir_base)
    for a, b in zip(sorted(expected_file_list), sorted(actual_file_list)):
        assert a == b
    assert len(set(expected_file_list) & set(actual_file_list)) == len(expected_file_list)


def test_parse_file_pairs():
    file_list = [
        "/path/to/a.txt",
        "/path/to/a.jpg",
        "path/to/a.txt",
        "path/to/a.jpg",
        "/path/to/b.txt",
        "/path/to/b.jpg",
        "/path/to/b.mp3"
    ]

    expected_pairs = {
        "/path/to/a": {".jpg": True, ".txt": True},
        "path/to/a": {".jpg": True, ".txt": True},
        "/path/to/b": {".jpg": True, ".txt": True, ".mp3": True}
    }

    assert expected_pairs == parse_file_pairs(file_list)


@pytest.mark.parametrize("fn_input,expected", [
    ("/path/to/file.txt", ("/path/to", "file", ".txt")),
    ("file.txt", ("", "file", ".txt")),
])
def test_file_parts(fn_input, expected):
    base_dir, base_fn, ext = file_parts(fn_input)
    exp_base_dir, exp_base_fn, exp_ext = expected
    assert base_dir == exp_base_dir
    assert base_fn == exp_base_fn
    assert ext == exp_ext


def test_previous_fail_case_do(fail_case_file_list):



import sys


def greet_world(descriptor=None):
    if descriptor:
        sys.stdout.write("Hello, {} World!\n".format(descriptor))
    sys.stdout.write("Hello, World!\n")


if __name__ == '__main__':
    greet_world()
